package nl.uvt.locator;

import java.io.*;
import java.net.*;
import java.math.*;
import java.security.*;
import java.util.*;
import java.util.regex.*;

import nu.xom.*;

public class LinkDbInfo extends LinkInfo {

	List<Threshold> thresholds;

	LinkDbInfo(){
		thresholds = new LinkedList<Threshold>();
	}

	public boolean inRange( QueryData pQ) throws LocatorException{
		return Threshold.inRange( thresholds, pQ );
	}


	void fill( Node rec ){
		String res = getValue( rec, "parameters/issn");
		if ( res.length() == 0 ){
			res = getValue( rec, "issn");		
		}
		if ( res.length() > 0 ){
			Pattern issnPat = Pattern.compile("([\\dX]*)");
			Matcher issnMatcher = issnPat.matcher(res);
			if ( issnMatcher.find() ){
				linkIssn = issnMatcher.group();
//				logger.debug( "matched!"+issn);
			}
		}
		res = getValue( rec, "title" );
		if ( res.length() > 0 )
			linkTitle = res;
		res = getValue( rec, "url");
		if ( res.length() > 0 )
			linkUrl = res;
		res = getValue( rec, "template" );
		if ( res.length() > 0 )
			linkMethod = res;
		res = getValue( rec, "publisher");
		if ( res.length() > 0 )
			linkPublisher = res;
		Nodes nodes = rec.query( "threshold ");
		for ( int i=0; i < nodes.size(); ++i ){
			Threshold th = new Threshold();
			th.fill( nodes.get(i) );
			thresholds.add( th );
		}
	}

	public String toString(){
		String result = "LinkDbInfo:";
		result += " issn='" + linkIssn;
		result += "' title='" + linkTitle;
		result += "' url='" + linkUrl;
		result += "' template='" + linkMethod;
		result += "' publisher='" + linkPublisher;
		result += "' thresholds=";
		for ( int i=0; i < thresholds.size(); ++i ){
			result += thresholds.get(i).toString();
		}
		return result;
	}

	public boolean applyMethod( QueryData pQ ){
/* 
 	RANKINGS:
	 	1.  pdf/html
	 	2.  artikelpagina (abstract o.i.d. + link naar full text)
		3.  issuepagina (bevat overzicht artikelen)
		4.  volumepagina (bevat overzicht issues)
		5.  tijdschriftpagina (bevat overzicht volumes)
		6.  site aanbieder (bevat overzicht tijdschriften of zoekbox)
*/
		boolean notYet= true;
		boolean doiTried = false;
		rank = 6; // fear the worst
		if ( isDOImethod( linkMethod ) ){
			logger.debug("Method '" + linkMethod + "' is a well-known DOI method");
			String myUrl = generateDOI( pQ );
			doiTried = true;
			if ( !myUrl.equals("") ){
				notYet = false;
				rank = 1;
				generatedUrl = myUrl;
			}
		}
		if ( notYet ) {
			logger.debug("Method= " + linkMethod );
			if ( linkMethod.equals("ABI") ) {
				String newUri = "http://gateway.proquest.com/openurl?ctx_ver=Z39.88-2003&res_id=xri:pqd&rft_val_fmt_=ori:fmt:kev:mtx:journal&issn=";
				newUri += linkIssn;
				newUri += "&jtitle=" + linkTitle;
				if ( pQ.atitle.length() > 0 ){
					try {
						newUri += "&atitle=" + URLEncoder.encode( pQ.atitle, "ISO-8859-1" );
					} catch (UnsupportedEncodingException e) {
						logger.fatal( "encoding title failed", e );
					}
				}
				if ( pQ.thVolume > 0 )
					newUri += "&volume=" + pQ.thVolume;
				if ( !pQ.issue.equals( "" ) ){
					newUri += "&issue=" + pQ.issue;
				}
				else if ( pQ.thIssue > 0 )
					newUri += "&issue=" + pQ.thIssue;
				if ( !pQ.spage.equals("") ){
					newUri += "&spage=" + pQ.spage;
					rank = 1;
				}
				else 
					rank = 2;
				generatedUrl = newUri;
			}
			else if ( linkMethod.equals("ACM") ) {
				rank = 5;
				generatedUrl = linkUrl;
			}
			else if ( linkMethod.equals("AMA") ) {
				String newUri = linkUrl + "/cgi/reprint/";
				newUri += getRequired( linkMethod, "volume", pQ.volume );
				newUri += "/" + getRequired( linkMethod,  "issue", pQ.issue );
				if ( !pQ.spage.equals("") ){
					newUri += "/" + pQ.spage;
					rank = 1;
				}
				generatedUrl = newUri;
			}
			else if ( linkMethod.equals("APP") ) {
				String newUri = linkUrl;
				if ( pQ.thIssue >= 0 && pQ.thVolume >= 0 ){
					if ( !pQ.spage.equals("") ){
						newUri += "/cgi/content/full";
						newUri += "/" + pQ.thVolume;
						newUri += "/" + pQ.thIssue + "/" + pQ.spage;
						rank = 1;
					}
					else {
						newUri += "/content/vol" + pQ.thVolume;
						newUri += "/issue" + pQ.thIssue;
						rank = 3;
					}
				}
				else {
					rank = 6;
				}
				generatedUrl = newUri;
			}
			else if ( linkMethod.equals("ASA") ) {
				// DOI failed
				if ( pQ.spage.equals("") ){
					generatedUrl = "http://pubs.amstat.org/toc/jasa";
					generatedUrl += "/" + getRequired( linkMethod,  "volume", pQ.volume );
					generatedUrl += "/" + getRequired( linkMethod,  "issue", pQ.issue );
					rank = 3;
				}
				else {
					generatedUrl = linkUrl;
					rank = 6;
				}
			}
			else if ( linkMethod.equals("Berkeley") ) {
				generatedUrl = linkUrl;
				generatedUrl += "/vol" + getRequired( linkMethod,  "volume", pQ.volume );
				generatedUrl += "/iss" + getRequired( linkMethod,  "issue", pQ.issue );
				rank = 2;
			}
			else if ( linkMethod.equals("Bernouilli") ) {
				rank = 6;
				generatedUrl = linkUrl;
			}
			else if ( linkMethod.equals("BOOM") ) {
				rank = 5;
				generatedUrl = linkUrl;
			}
			else if ( linkMethod.equals("catchword") ) {
				String newUri = ""; 
				if ( pQ.issue.equals("")  || pQ.thVolume <= 0  || pQ.spage.equals("") ){
					newUri = linkUrl;
					rank = 5;
				}
				else {
					newUri = "http://www.ingentaselect.com/rpsv/cgi-bin/cgi?body=linker&reqidx=";
					newUri += getRequired( linkMethod,  "issn", linkIssn, pQ.issn );
					newUri += "(" + getRequired( linkMethod,  "year", pQ.year) + ")L.";
					newUri += pQ.spage;
					rank = 1;
				}
				generatedUrl = newUri;
			}
			else if ( linkMethod.equals("Clark") ) {
				generatedUrl = linkUrl;
				generatedUrl += "issue.cfm?issue=" + pQ.year + "_0";
				generatedUrl +=  getRequired( linkMethod, "issue", pQ.issue );
				generatedUrl += ".html"; 
				rank = 3;
			}
			else if ( linkMethod.equals("CUP") ) {
				rank = 5;
				generatedUrl = linkUrl;
			}
			else if ( linkMethod.equals("DOAJ") ) {
				rank = 6;
				generatedUrl = linkUrl;
			}
			else if ( linkMethod.equals("DUP") ) {
				//when DOI failed.
				String newUri = linkUrl; 
				if ( pQ.thVolume <= 0  ){
					rank = 5;
				}
				else {
					rank = 1;
					newUri += "/cgi/search?author1=" + pQ.aulast;
					newUri += "&pubdate_year="+ pQ.year;
					newUri += "&volume="+ pQ.thVolume;
					newUri += "&firstpage="+ pQ.spage;
					if ( pQ.spage.equals("") )
						++rank;
					if ( pQ.aulast.equals("") )
						++rank;	
				}
				generatedUrl = newUri;
			}
			else if ( linkMethod.equals("EBSCO") ) {
				String openURL = linkUrl;
				rank = 5;
				if ( !pQ.volume.equals("") ){
					String issn = getRequired( linkMethod,  "issn", linkIssn, pQ.issn );
					int missing = 0;
					if ( issn.equals(""))
						missing++;
					if ( pQ.issue.equals(""))
						missing++;
					if ( pQ.spage.equals(""))
						missing++;
					if ( pQ.title.equals("") || pQ.atitle.equals(""))
						missing++;
					if ( pQ.issue.equals("") )
						missing++;
					if ( missing <= 1 ){
						openURL = "http://openurl.ebscohost.com/linksvc/linking.aspx";
						openURL += "?genre=" + pQ.genre;
						openURL += "&issn=" + issn;
						openURL += "&issue=" + pQ.issue;
						if ( pQ.title.length() > 0 ){
							try {
								openURL += "&title=" + URLEncoder.encode( pQ.title, "ISO-8859-1" );
							} catch (UnsupportedEncodingException e) {
								logger.fatal( "encoding title failed", e );
							}
						}
						openURL += "&volume=" + pQ.volume;
						openURL += "&spage=" + pQ.spage;
						if ( pQ.atitle.length() > 0 ){
							try {
								openURL += "&atitle=" + URLEncoder.encode( pQ.atitle, "ISO-8859-1" );
							} catch (UnsupportedEncodingException e) {
								logger.fatal( "encoding atitle failed", e );
							}
						}
						rank = 2;
					}
				}
				generatedUrl = openURL;
			}
			else if ( linkMethod.equals("Euclid") ) {
				rank = 6;
				generatedUrl = linkUrl;
			}
			else if ( linkMethod.equals("Extenza") ) {
				String newUri = linkUrl;
				if ( pQ.thVolume <= 0 || pQ.thIssue <= 0 ){
					rank = 5;
				}
				else {
					newUri = "http://www.atypon-link.com/AEAP/toc/aer/" + pQ.thVolume + "/" + pQ.thIssue;
					rank = 3;
				}
				generatedUrl = newUri;
			}
			else if ( linkMethod.equals("Hart") ) {
				rank = 1;
				generatedUrl = linkUrl;
				generatedUrl += "/volumes/" + getRequired( linkMethod, "volume", pQ.volume );
				generatedUrl += "/issues/" + getRequired( linkMethod, "issue", pQ.issue );
			}
			else if ( linkMethod.equals("HeinOnline") ) {
				rank = 5;
				generatedUrl = linkUrl;
			}
			else if ( linkMethod.equals("Highwire") ) {
				if ( pQ.spage.equals("")){
					rank = 3; 
					generatedUrl = linkUrl + "/content/";
					generatedUrl += "vol"+ getRequired( linkMethod, "volume", pQ.volume );
					generatedUrl += "/issue" + getRequired( linkMethod, "issue", pQ.issue );
				}
				else {
					rank = 1;
					generatedUrl = linkUrl + "/cgi/content/full/";
					generatedUrl += getRequired( linkMethod, "volume", pQ.volume );
					generatedUrl += "/" + getRequired( linkMethod, "issue", pQ.issue );
					generatedUrl += "/" + getRequired( linkMethod, "spage", pQ.spage );
				}
			}
			else if ( linkMethod.equals("IBDF") ) {
				rank = 5;
				generatedUrl = linkUrl;
			}
			else if ( linkMethod.equals("IngentaConnect") ) {
				String openURL = ""; 
				if ( pQ.issue.equals("")  || pQ.thVolume <= 0 || pQ.spage.equals("") ){
					openURL = linkUrl;
					rank = 5;
				}
				else {
					openURL = "http://openurl.ingenta.com/content?";
					String myIssn = getRequired( linkMethod,  "issn", linkIssn, pQ.issn );
					if ( !myIssn.equals("") ){
						myIssn = myIssn.substring(0, 4) + "-" + myIssn.substring(4,8);
					}
					openURL += "?genre=" + pQ.genre;
					openURL += "&issn=" + myIssn;
					openURL += "&issue=" + pQ.issue;
					openURL += "&volume=" + pQ.volume;
					openURL += "&spage=" + pQ.spage;
					rank = 1;
				}
				generatedUrl = openURL;
			}
			else if ( linkMethod.equals("IPO") ) {
				rank = 6;
				generatedUrl = linkUrl;
			}
			else if ( linkMethod.equals("Journal")  ) {
				rank = 6;
				generatedUrl = linkUrl;
			}
			else if ( linkMethod.equals("JSTOR")  ) {
				String myUrl = "http://links.jstor.org/sici?";
				myUrl += "sici=" + getRequired( linkMethod, "issn", linkIssn );
				myUrl += "(" + getRequired( linkMethod, "year", pQ.year ) + ")";
				if ( pQ.thVolume > 0 )
					myUrl += pQ.thVolume;
				if ( pQ.thIssue > 0 )
					myUrl += ":" + pQ.thIssue;
				if ( pQ.spage.equals("")){
					myUrl += "%3C%3E2.0.CO;2-B";
					rank = 3;
				}
				else {
					myUrl += "%3C" + pQ.spage + "%3E2.0.CO;2-B";
					rank = 1;
				}
				generatedUrl = myUrl;
			}
			else if ( linkMethod.equals("KLI")  ) {
				generatedUrl = "http://www.kluwerlawonline.com/toc.php?area=Journals&mode=bypub&level=6&values=Journals~~";
				generatedUrl += getRequired( linkMethod,  "title", linkTitle, pQ.title );
				generatedUrl += "~Volume+" + getRequired( linkMethod,  "volume", pQ.volume );
				generatedUrl += "+(" + getRequired( linkMethod,  "year", pQ.year );
				generatedUrl += ")~Issue+" + getRequired( linkMethod,  "issue", pQ.issue );
				rank = 3;
			}
			else if ( linkMethod.equals("Login-Hint")  ) {
				rank = 6;
				generatedUrl = linkUrl;
			}
			else if ( linkMethod.equals("Metapress")  ) {
				String openURL = "http://www.metapress.com/openurl.asp";
				if ( !pQ.genre.equals("") )
					openURL += "?genre=" + pQ.genre;
				String issn = "";
				if ( !linkIssn.equals("") )
					issn = linkIssn;
				else if ( !pQ.issn.equals("") )
					issn= pQ.issn;
				if ( !issn.equals("") ){
					openURL += "&issn=" + issn;
					if ( !pQ.issue.equals("") ){
						if ( pQ.thVolume > 0 ) {
							openURL += "&issue=" + pQ.issue;
							openURL += "&volume=" + pQ.volume;
							if ( !pQ.spage.equals("") ){
								openURL += "&spage=" + pQ.spage;
								rank = 1;
							}
							else
								rank = 3;
						}
						else
							rank = 5;
					}
					else
						rank = 5;
				}
				else {
					openURL = "http://www.metapress.com/journals/";
					rank =6; 
				}
				generatedUrl = openURL;
			}
			else if ( linkMethod.equals("MLCV")  ) {
				String spage = lPad( pQ.spage, '0', 4 );
				String volume = lPad( pQ.volume, '0', 3 );
				String part = linkUrl;
				rank = 6;
				if ( part.length() > 36 ){
					part = part.substring(36);
					String[] splits = part.split("/");
					if ( splits.length > 1 ){
						rank = 1;
						generatedUrl = "http://www.multilingual-matters.net/" + splits[0] + "/" 
						+ volume + "/" + spage + "/" + splits[0] + volume + spage + ".pdf";
					}
				}
			}	
			else if ( linkMethod.equals("NASW") ) {
				rank = 5;
				generatedUrl = linkUrl;
			}
			else if ( linkMethod.equals("NBLC") ) {
				rank = 1;
				generatedUrl = "http://www.leesgoedplus.nl/archief/";
				generatedUrl += getRequired( linkMethod, "title", linkTitle ) + "/";
				generatedUrl += getRequired( linkMethod, "year", pQ.year ) + "/";
				generatedUrl += getRequired( linkMethod, "issue", pQ.issue ) + ".pdf";
			}
			else if ( linkMethod.equals("NCTE") ) {
				rank = 6;
				generatedUrl = linkUrl;
			}
			else if ( linkMethod.equals("OLC") ) {
				// OLC is not handled by us!
				// should link to the docserver
				diagnostics.add( new Diagnostic( 1, "method OLC is not handled by the Locator", "docserver link needed"));
			}
			else if ( linkMethod.equals("ovid") ) {
				String newUri = "http://gateway.ovid.com/ovidweb.cgi?T=JS&PAGE=fulltext&D=ovft&MODE=ovid&NEWS=N&SEARCH=";
				newUri += linkIssn.substring(0, 4) + "-" + linkIssn.substring(4,8);
				newUri += ".is+and+";
				newUri += getRequired( linkMethod,  "volume", pQ.volume );
				newUri += ".vo+and+";
				newUri += getRequired( linkMethod,  "issue", pQ.issue );
				newUri += ".ip+and+" + getRequired( linkMethod,  "spage", pQ.spage );
				if ( diagnostics.isEmpty() ){
					newUri += ".pg&LOGOUT=n";
					rank = 1;
					generatedUrl = newUri;
				}
				else {
					generatedUrl = linkUrl;					
					rank = 5;
				}
			}
			else if ( linkMethod.equals("Peeters") ) {
				int pos = linkUrl.indexOf("code=");
				String code = linkUrl.substring(pos+5);
				String newUri = "http://poj.peeters-leuven.be/issue.php?journal_code=" + code;
				newUri += "&issue=" + getRequired( linkMethod,  "issue", pQ.thIssue );
				newUri += "&vol=" + getRequired( linkMethod,  "volume", pQ.volume);
				rank = 1;
				generatedUrl = newUri;
			}
			else if ( linkMethod.equals("PPP") ) {
				rank = 6;
				generatedUrl = linkUrl;
			}
			else if ( linkMethod.equals("RCP") ) {
				rank = 6;
				generatedUrl = linkUrl;
			}
			else if ( linkMethod.equals("SAGE") ) {
				if ( pQ.spage.equals("")) {
					generatedUrl = linkUrl + "/content/vol"	
					+ getRequired( linkMethod,  "volume", pQ.volume ) + "/issue" 
					+ getRequired( linkMethod,  "issue", pQ.issue ) + "/" ;
					rank = 3;
				}
				else {
					generatedUrl = linkUrl + "/cgi/reprint/"	
					+ getRequired( linkMethod,  "volume", pQ.volume ) + "/" 
					+ getRequired( linkMethod,  "issue", pQ.issue ) + "/" 
					+ getRequired( linkMethod,  "spage", pQ.spage ) + ".pdf";
					rank = 1;
				}
			}
			else if ( linkMethod.equals("SD")){
				String myUrl = "";
				if ( !(pQ.thVolume == 0 )
						&& !pQ.spage.equals("") ){
					logger.debug( "create md5 hashed SD link");
					String SDorigin="UTILBURG";
					String SDsalt = "(CxZ:=Jrxs;hXYWS!KVpdj!jgeO7NSdJ";
					String volKey = linkIssn;
					volKey += "#" + getRequired( linkMethod, "volume", pQ.volume );
					volKey += "#" + pQ.spage;
					if ( pQ.thIssue >= 0 )
						volKey += "#" + pQ.thIssue;
					String md5source = "_ob=GatewayURL&_origin=" + SDorigin 
					+ "&_method=citationSearch&_volkey=" + volKey
					+ "&_version=1" + SDsalt;
//					logger.debug("md5:'" + md5source + "'" );
					MessageDigest md5;
					String md5Hash = "";
					try {
						md5 = MessageDigest.getInstance("MD5");
						md5.update(md5source.getBytes(), 0, md5source.length());
						byte mdBytes[] = md5.digest();
						md5Hash = new BigInteger(1,mdBytes).toString(16);
						if ( md5Hash.length() < 32 )
							md5Hash = '0' + md5Hash; 
//						logger.debug("md5 hash:'" + md5Hash + "'" );
					} catch (NoSuchAlgorithmException e) {
						logger.fatal("MD5 hasher", e);
					}
					try {
						myUrl = "http://www.sciencedirect.com/science?_ob=GatewayURL&_origin="
							+ SDorigin + "&_method=citationSearch&_volkey="
							+ URLEncoder.encode(volKey, "ISO-8859-1" ) + "&_version=1&md5=" + md5Hash;
					} catch (UnsupportedEncodingException e) {
						logger.fatal( "encoding failed", e );
					}
					rank = 1;
				}
				if ( myUrl.equals("") ) {
					logger.debug( "last resort: linkDb link");
					rank = 5;
					myUrl = linkUrl;
				}
				generatedUrl = myUrl;
			}
			else if ( linkMethod.equals("springerlink") ) {
				// springerlink does openUrl, but the linkDb link already gives genre and issn
				String openURL = linkUrl;
				rank = 5;
				if ( !pQ.volume.equals("") ){
					if ( !pQ.issue.equals("")){
						if ( !pQ.spage.equals("") ){
							openURL += "&volume=" + pQ.volume + "&issue=" + pQ.issue + "&spage=" + pQ.spage;
							rank = 1;
						}
						else {
							openURL += "&volume=" + pQ.volume + "&issue=" + pQ.issue;
							rank = 3;

						}
					}
				}
				generatedUrl = openURL;
			}
			else if ( linkMethod.equals("swetsnet") ) {
				String newUri = "http://www.swetsnet.nl/link/access_db?issn=";
				newUri += linkIssn;
				newUri += "&vol=";
				newUri += pQ.volume; 
				newUri += "&iss=";
				newUri += pQ.issue;
				newUri += "&page=" +pQ.spage;
				newUri += "&year=" + pQ.year + "&FT=1";
				rank = 1;
				generatedUrl = newUri;
			}
			else if ( linkMethod.equals("TF")  ) {
				if ( pQ.spage.equals("")){
					rank = 3;
				}
				else {
					rank = 1;
				}
				generatedUrl = "http://www.informaworld.com/openurl?" + generateOpenURL( pQ );
			}
			else if ( linkMethod.equals("VenR")  ) {
				rank = 6;
				generatedUrl = linkUrl;
			}
			else if ( linkMethod.equals("WESTLAW") ) {
				rank = 6;
				generatedUrl = linkUrl;
			}
			else if ( linkMethod.equals("WL") ) {
				rank = 6;
				generatedUrl = linkUrl;
				generatedUrl += "/PasswordPDF/Vol" + getRequired( linkMethod, "volume", pQ.volume );
				generatedUrl += "no" + getRequired( linkMethod,  "issue", pQ.issue );
				generatedUrl += "_" + getRequired( linkMethod,  "year", pQ.year );
			}
			else if ( linkMethod.equals("WPNR") ) {
				rank = 6;
				generatedUrl = linkUrl;

			}
			else {
				String myUrl = "";
				if ( !doiTried ){
					// default handling is DOI
					logger.debug( "try DOI" );
					myUrl = generateDOI( pQ );
					if ( !myUrl.equals("") ){
						logger.info("the method " + linkMethod + " gave a DOI, maybe it's a good advise to add it to DOImethods in your config?");
						rank = 1;
					}
				}
				if ( myUrl.equals("") ){
					rank = 6;
					myUrl = linkUrl;
				}
				generatedUrl = myUrl;
			}
		}
		if ( !diagnostics.isEmpty() ){
			logger.warn("Link DB Diagnostics: " );
			logger.warn("query was: " + pQ );
			for( int i=0; i < diagnostics.size(); ++i )
				logger.warn( diagnostics.get(i).toString() );
			diagnostics.clear();
			generatedUrl = linkUrl;
		}
		logger.debug("generated URL: "+ generatedUrl );
		return generatedUrl != "";
	}

	private boolean isDOImethod(String linkMethod) {
		if ( Locator.DOImethods == null ){
			Locator.DOImethods = new HashSet<String>();
			String doiM = Locator.locatorConfig.getProperty("DOImethods","");
			String [] doiS = doiM.split(",");
			for ( int i=0; i < doiS.length; ++i ){
				Locator.DOImethods.add(doiS[i].trim());
			}
			logger.debug("Known DOI methods: " + Locator.DOImethods.toString() );
		}
		if ( Locator.DOImethods.contains(linkMethod) )
			return true;
		else
			return false;
	}

	private String lPad(String string, char c, int len ) {
		// pad string with c to get length len
		String result = string;
		while ( result.length() < len ) result = c + result;
		return result;
	}

	private int getRequired( String linkMethod, String string, int val ) {
		if	( val >= 0 )
			return val;
		else
			diagnostics.add(misPar(linkMethod, string) );
		return -1;
	}

	private String getRequired( String linkMethod, String string, String val ) {
		if	( !val.equals("") )
			return val;
		else
			diagnostics.add(misPar(linkMethod,string) );
		return "";
	}

	private String getRequired( String linkMethod, String string, String lVal, String qVal ) {
		if	( !lVal.equals("") )
			return lVal;
		else if ( !qVal.equals("") )
			return qVal;
		else
			diagnostics.add(misPar(linkMethod, string) );
		return "";
	}


	private Diagnostic misPar( String method, String par ){
		return new Diagnostic( 10, method+": missing required parameter", par );
	}


	private String generateDOI( QueryData pQ ){
		String crossrefUri = Locator.locatorConfig.getProperty("crossrefUri");
		try {
			Properties prop = new Properties();
			String qData = "";
			qData += linkIssn + "|";
//			if ( linkTitle.length() > 0 )
//			qData +=  URLEncoder.encode(linkTitle, "ISO-8859-1" ) +"||";
//			else
			qData += pQ.atitle +"||";
			qData += pQ.thVolume + "|";
			if ( !pQ.issue.equals(""))
				qData += pQ.issue + "|";
			else
				qData += "|";
			qData += pQ.spage + "|";
			qData += pQ.year + "|||";
			prop.setProperty("qdata", qData );
			prop.setProperty( "pwd", "crpw021");
			prop.setProperty( "usr", "tuli");
			String url = HttpConnection.createUrlString(crossrefUri, prop); 
			InputStream result = HttpConnection.HttpGetInputStream( url, null );
			BufferedReader in = new BufferedReader( new InputStreamReader(result) );
			logger.debug( "crossref returns:");
			String doi = "";
			while ( in.ready() ){
				String line = in.readLine();
				if ( line.contains("|") ){
					String [] arr = line.split( "\\|" );
					if ( arr.length > 8 && arr[9] != "" )
						doi = "http://dx.doi.org/" +  URLEncoder.encode(arr[9], "ISO-8859-1" );
				}
				logger.debug( line );
			}
			return doi;
		}
		catch (Exception e) 
		{
			logger.debug( "crossref failed, uri" );
			return "";
		}
	}


}
